#include "MainWindow.h"
#include "ui_MainWindow.h"

#include <QSettings>
#include <QMessageBox>
#include <QProgressDialog>
#include <QHBoxLayout>
#include <QGroupBox>
#include <QCheckBox>
#include <QCoreApplication>
#include <QtDebug>
#include <QLineEdit>
#include <QAction>

#include "lqmeas/log/Log.h"
#include "lqmeas/log/LogPanel.h"
#include "lqmeas/StdErrors.h"
#include "lqmeas/devs/LTR/modules/LTR35/LTR35Info.h"
#include "lqmeas/devs/LTR/modules/LTR35/LTR35Config.h"
#include "lqmeas/devs/LTR/modules/LTR35/LTR35OutConfig.h"
#include "lqmeas/devs/LTR/crates/LTRCrate.h"
#include "lqmeas/devs/LTR/LTRResolver.h"
#include "lqmeas/ifaces/out/Signals/Analog/Sin/AnalogSignalSin.h"
#include "lqmeas/ifaces/out/Signals/Analog/Sin/AnalogSignalSinConfig.h"
#include "lqmeas/ifaces/out/Signals/Analog/Const/AnalogSignalConst.h"
#include "lqmeas/ifaces/out/Signals/Analog/Const/AnalogSignalConstConfig.h"
#include "lqmeas/ifaces/out/Signals/Analog/Pulse/AnalogSignalPulse.h"
#include "lqmeas/ifaces/out/Signals/Analog/Pulse/AnalogSignalPulseConfig.h"
#include "lqmeas/ifaces/out/Signals/Discrete/Pulse/DiscreteSignalPulse.h"
#include "lqmeas/ifaces/out/Signals/Discrete/Pulse/DiscreteSignalPulseConfig.h"
#include "lqmeas/ifaces/out/Signals/Discrete/SinglePulseBurst/DiscreteSignalSinglePulseBurst.h"
#include "lqmeas/ifaces/out/Signals/Discrete/SinglePulseBurst/DiscreteSignalSinglePulseBurstConfig.h"
#include "LTRModuleSelectionUI/LTRModuleSelectionUI.h"

#include "QTranslationSwitch.h"
#include "WaitDialog.h"
#include <math.h>

#include "ConfigDialog.h"


#ifndef M_PI
    #define M_PI       3.14159265358979323846
#endif

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow) {

    ui->setupUi(this);
    setWindowTitle(QCoreApplication::applicationName() + " - " + QCoreApplication::applicationVersion());

    connect(ui->resetButton, &QPushButton::pressed, this, &MainWindow::reset);

    LQMeas::LogPanel *logPanel = new LQMeas::LogPanel();
    addDockWidget(Qt::DockWidgetArea::BottomDockWidgetArea, logPanel);
/* загрузка настроек состояния интерфейса */
    QTranslationSwitch &ts {QTranslationSwitch::instance()};
    ts.init(PRJ_TRANSLATIONS_DIR);
    ts.addTranslationFile(LQMEAS_TS_BASENAME);
    ts.setLanguage(QTranslationSwitch::langNameRu());
    LQMeasLog->retranslate();

    QSettings set;
    set.beginGroup("MainWindow");
    restoreState(set.value("state").toByteArray());
    resize(set.value("size", QSize(800, 600)).toSize());
    move(set.value("pos", QPoint(100, 200)).toPoint());
    set.endGroup();

    set.beginGroup("DeviceConfig");
    set.beginGroup("Afc");
    m_afcCfg.load(set);
    set.endGroup();
    set.endGroup();

    m_curCfg = new LQMeas::LTR35Config();

    LQMeasLog->info(tr("Приложение запущено"));


    setManualUi();

    ui->dacFreqStd->addItem(tr("192 КГц"), QVariant::fromValue(LQMeas::LTR35OutConfig::OutStdGenFreq::Freq_192K));
    ui->dacFreqStd->addItem(tr("96 КГц"), QVariant::fromValue(LQMeas::LTR35OutConfig::OutStdGenFreq::Freq_96K));
    ui->dacFreqStd->addItem(tr("48 КГц"), QVariant::fromValue(LQMeas::LTR35OutConfig::OutStdGenFreq::Freq_48K));

    ui->syncStartMode->addItem(tr("Внутренняя"), QVariant::fromValue(LQMeas::LTR35SyncConfig::SyncStartMode::Internal));
    ui->syncStartMode->addItem(tr("Мастер"), QVariant::fromValue(LQMeas::LTR35SyncConfig::SyncStartMode::Master));
    ui->syncStartMode->addItem(tr("Подчиненный"), QVariant::fromValue(LQMeas::LTR35SyncConfig::SyncStartMode::Slave));


    ui->slaveSyncSrc->addItem(tr("Фронт DI2"), QVariant::fromValue(LQMeas::LTR35SyncConfig::SyncSlaveSrc::DI2_Rise));
    ui->slaveSyncSrc->addItem(tr("Спад DI2"), QVariant::fromValue(LQMeas::LTR35SyncConfig::SyncSlaveSrc::DI2_Fall));


    loadSignalSettings();

    m_devsel = new LTRModuleSelectionUI{QStringList {LQMeas::LTR35TypeInfo::defaultInfo().deviceTypeName()},
                                        ui->ltr35_crate, ui->ltr35_slot, this};

    connect(m_devsel, &LTRModuleSelectionUI::beforeModuleChange, this, &MainWindow::onBeforeModuleChanged);
    connect(m_devsel, &LTRModuleSelectionUI::moduleChanged, this, &MainWindow::onModuleChanged);
    connect(ui->dacFreqManualEn, &QCheckBox::stateChanged,  this, &MainWindow::manualUpdateUi);
    connect(ui->dacModeBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
            this, &MainWindow::manualUpdateUi);

    connect(ui->disChsButton, &QPushButton::clicked, this, &MainWindow::disableChannels);
    connect(ui->cfgButton, &QPushButton::clicked, this, &MainWindow::showConfigDialog);


    addAction(ui->actionSetSignal);
    connect(ui->actionSetSignal,  &QAction::triggered, this, &MainWindow::onSetSigButtonClicked);
    connect(ui->setSigButton, &QPushButton::clicked, this, &MainWindow::onSetSigButtonClicked);
    manualConnectSignals(true);
    connect(ui->dacModeBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
            this, &MainWindow::uiEnStateUpdate);
    connect(ui->syncStartMode, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
            this, &MainWindow::uiEnStateUpdate);
    connect(ui->echoEn, &QCheckBox::stateChanged, this, &MainWindow::uiEnStateUpdate);
    uiEnStateUpdate();
}



MainWindow::~MainWindow() {
    delete ui;
}

void MainWindow::reset() {
    LQMeasLog->info("Сброс программы...");
    closeCurModule();
    m_devsel->reset();
}

void MainWindow::onBeforeModuleChanged() {
    closeCurModule();
}

void MainWindow::onModuleChanged() {
    QSharedPointer<LQMeas::LTR35> dac = curDev();

    if (dac) {
        const LQMeas::LTR35Info &info = dac->devspecInfo();
        ui->ltr35_serial->setText(info.serial());
        ui->ltr35_modification->setText(info.type().deviceModificationName());
        ui->ltr35_verPLD->setText(info.pldVerStr());
        ui->ltr35_verFPGA->setText(info.fpgaVerStr());

        m_curCfg->updateDevTypeInfo(&dac->devInfo().type());
        applyAfcParams();
    } else {
        ui->ltr35_serial->clear();
        ui->ltr35_modification->clear();
        ui->ltr35_verPLD->clear();
        ui->ltr35_verFPGA->clear();
    }
    manualUpdateUiDev();
}

void MainWindow::closeCurModule() {
    QSharedPointer<LQMeas::LTR35> dac = curDev();
    if (dac) {
        WaitDialog dialog(this);
        dialog.setModal(true);
        dialog.show();
        if (dac->outSyncGenRunning()) {
            LQError stop_err;
            dialog.setText(tr("Остановка генерации..."));
            dac->outSyncGenStopRequest(stop_err);
            qApp->processEvents();
            dac->outSyncGenStop(stop_err);
        }
        dialog.setText(tr("Закрытие модуля..."));
        qApp->processEvents();
        m_devsel->closeCurModule();

        dialog.close();
    }
    ui->SignalLed->setChecked(false);
}




MainWindow::ChUi MainWindow::createChannelUi(QGridLayout *lout, QString name, int row,
                                             bool allCh) {
    ChUi ch_ui;
    QString specVal = allCh ? "-" : QString();

    ch_ui.enBox = new QCheckBox(name);
    ch_ui.enBox->setMaximumWidth(150);
    lout->addWidget(ch_ui.enBox, row, 0);

    ch_ui.typeBox = new QComboBox();
    ch_ui.typeBox->addItem(LQMeas::AnalogSignalSin::instance().displayName());
    ch_ui.typeBox->addItem(LQMeas::AnalogSignalConst::instance().displayName());
    ch_ui.typeBox->addItem(LQMeas::AnalogSignalPulse::instance().displayName());

    ch_ui.typeBox->setCurrentIndex(1);


    lout->addWidget(ch_ui.typeBox, row, 1);


    ch_ui.ampType = new QComboBox();
    ch_ui.ampType->addItem(tr("Пик"), false);
    ch_ui.ampType->addItem(tr("RMS"), true);
    ch_ui.ampType->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));


    ch_ui.ampBox = new QDoubleSpinBox();
    ch_ui.ampBox->setSuffix(tr(" мВ"));
    ch_ui.ampBox->setDecimals(3);
    ch_ui.ampBox->setMaximum(20*1000);
    ch_ui.ampBox->setMinimum(-10*1000);
    ch_ui.ampBox->setValue(10*1000);
    ch_ui.ampBox->setGroupSeparatorShown(true);
    ch_ui.ampBox->setSpecialValueText(specVal);
    ch_ui.ampBox->setAlignment(Qt::AlignVCenter | Qt::AlignRight);


    QWidget *ampWgt = new QWidget();
    QHBoxLayout *ampWgtLout = new QHBoxLayout();
    ampWgtLout->setContentsMargins(0, 0, 0, 0);
    ampWgtLout->addWidget(ch_ui.ampBox);
    ampWgtLout->addWidget(ch_ui.ampType);
    ampWgt->setLayout(ampWgtLout);


    lout->addWidget(ampWgt, row, 2);


    ch_ui.offsBox = new QDoubleSpinBox();
    ch_ui.offsBox->setSuffix(tr(" мВ"));
    ch_ui.offsBox->setDecimals(3);
    ch_ui.offsBox->setMaximum(20*1000);
    ch_ui.offsBox->setMinimum(-10*1000);
    ch_ui.offsBox->setGroupSeparatorShown(true);
    ch_ui.offsBox->setSpecialValueText(specVal);
    ch_ui.offsBox->setAlignment(Qt::AlignVCenter | Qt::AlignRight);

    lout->addWidget(ch_ui.offsBox, row, 3);

    ch_ui.freqBox = new QDoubleSpinBox();
    ch_ui.freqBox->setSuffix(tr(" Гц"));
    ch_ui.freqBox->setDecimals(2);    
    ch_ui.freqBox->setMinimumWidth(150);
    ch_ui.freqBox->setMaximum(500000);
    ch_ui.freqBox->setMinimum(0);
    ch_ui.freqBox->setSpecialValueText(specVal);
    ch_ui.freqBox->setValue(19200);
    ch_ui.freqBox->setGroupSeparatorShown(true);
    ch_ui.freqBox->setAlignment(Qt::AlignVCenter | Qt::AlignRight);

    lout->addWidget(ch_ui.freqBox, row, 4);


    ch_ui.phaBox = new QDoubleSpinBox();
    ch_ui.phaBox->setSuffix(tr(" °"));
    ch_ui.phaBox->setDecimals(2);
    ch_ui.phaBox->setMinimumWidth(100);
    ch_ui.phaBox->setMaximum(360);
    ch_ui.phaBox->setMinimum(-360);
    ch_ui.phaBox->setSpecialValueText(specVal);
    ch_ui.phaBox->setValue(0);
    ch_ui.phaBox->setAlignment(Qt::AlignVCenter | Qt::AlignRight);

    lout->addWidget(ch_ui.phaBox, row, 5);

    ch_ui.dutyRatio = new QDoubleSpinBox();
    ch_ui.dutyRatio->setDecimals(3);
    ch_ui.dutyRatio->setMinimum(1);
    ch_ui.dutyRatio->setMaximum(10000);
    ch_ui.dutyRatio->setValue(2);
    ch_ui.dutyRatio->setSpecialValueText(specVal);
    ch_ui.dutyRatio->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
    lout->addWidget(ch_ui.dutyRatio, row, 6);


    ch_ui.rangeBox = new QComboBox();
    ch_ui.rangeBox->addItem("1:1");
    ch_ui.rangeBox->addItem("1:10");
    ch_ui.rangeBox->setCurrentIndex(0);
    lout->addWidget(ch_ui.rangeBox, row, 7);



    return ch_ui;
}

void MainWindow::setManualUi() {
    QGridLayout *lout = ui->ChannelSetLayout;
    int row = 1;

    for (unsigned ch=0; ch < dac_ch_max_cnt; ch++) {
        m_chUi.append(createChannelUi(lout, tr("Канал %1").arg(QString::number(ch+1)), row++));
    }

    lout->setRowMinimumHeight(row++, 20);
    m_allChUi = createChannelUi(lout, tr("Все каналы"), row++, true);

    connect(m_allChUi.enBox,   &QCheckBox::stateChanged, this, &MainWindow::allChEnChanged);
    connect(m_allChUi.freqBox, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
            this, &MainWindow::allChFreqValChanged);
    connect(m_allChUi.phaBox,  static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
            this, &MainWindow::allChPhaValChanged);
    connect(m_allChUi.dutyRatio, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
            this, &MainWindow::allChDutyValChanged);
    connect(m_allChUi.ampBox, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
            this, &MainWindow::allChAmpValChanged);
    connect(m_allChUi.offsBox, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
            this, &MainWindow::allChOffsValChanged);
    connect(m_allChUi.typeBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
            this, &MainWindow::allChSigTypeChanged);
    connect(m_allChUi.rangeBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
            this, &MainWindow::allChRangeChanged);
    connect(m_allChUi.ampType, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
            this, &MainWindow::allChAmpTypeChanged);
}

void MainWindow::manualUpdateUiDev() {
    QSharedPointer<LQMeas::LTR35> dac = curDev();
    ui->setSigButton->setEnabled(dac && dac->isOpened());
    ui->disChsButton->setEnabled(dac && dac->isOpened());


    double maxFreq = m_curCfg->out().outSyncGenFreq()/2;


    for (int ch=0; ch < m_chUi.size(); ch++) {
        m_chUi[ch].freqBox->setMaximum(maxFreq);
        m_chUi[ch].freqBox->setMinimum(0);
    }


    m_allChUi.freqBox->setMaximum(maxFreq);
    m_allChUi.freqBox->setMinimum(0);

    manualUpdateUi();


}

void MainWindow::manualUpdateChUi(int ch) {
    ChUi ch_ui =  ch >= 0 ? m_chUi[ch] : m_allChUi;


    if (ch >= m_curCfg->out().outInfo().outDacChannelsCnt()) {
        ch_ui.enBox->setEnabled(false);
        ch_ui.ampBox->setEnabled(false);
        ch_ui.ampType->setEnabled(false);
        ch_ui.typeBox->setEnabled(false);
        ch_ui.offsBox->setEnabled(false);
        ch_ui.freqBox->setEnabled(false);
        ch_ui.phaBox->setEnabled(false);
        ch_ui.dutyRatio->setEnabled(false);
        ch_ui.rangeBox->setEnabled(false);
    } else if (ch_ui.enBox->checkState()==Qt::Unchecked) {
        ch_ui.enBox->setEnabled(true);
        ch_ui.ampBox->setEnabled(false);
        ch_ui.ampType->setEnabled(false);
        ch_ui.typeBox->setEnabled(false);
        ch_ui.offsBox->setEnabled(false);
        ch_ui.freqBox->setEnabled(false);
        ch_ui.phaBox->setEnabled(false);
        ch_ui.dutyRatio->setEnabled(false);
        ch_ui.rangeBox->setEnabled(false);
    } else {
        QString type = ch_ui.typeBox->currentText();
        ch_ui.enBox->setEnabled(true);
        ch_ui.typeBox->setEnabled(true);
        ch_ui.rangeBox->setEnabled(true);

        if (type == LQMeas::AnalogSignalSin::instance().displayName()) {
             ch_ui.ampBox->setEnabled(true);
             ch_ui.ampType->setEnabled(true);
             ch_ui.offsBox->setEnabled(true);
             ch_ui.freqBox->setEnabled(true);
             ch_ui.phaBox->setEnabled(true);
             ch_ui.dutyRatio->setEnabled(false);
        } else if (type == LQMeas::AnalogSignalConst::instance().displayName()) {
            ch_ui.ampBox->setEnabled(true);
            ch_ui.ampType->setEnabled(false);
            ch_ui.offsBox->setEnabled(false);
            ch_ui.freqBox->setEnabled(false);
            ch_ui.phaBox->setEnabled(false);
            ch_ui.dutyRatio->setEnabled(false);
        } else if (type == LQMeas::AnalogSignalPulse::instance().displayName()) {
            ch_ui.ampBox->setEnabled(true);
            ch_ui.ampType->setEnabled(false);
            ch_ui.offsBox->setEnabled(true);
            ch_ui.freqBox->setEnabled(true);
            ch_ui.phaBox->setEnabled(false);
            ch_ui.dutyRatio->setEnabled(true);
        } else {
            ch_ui.ampBox->setEnabled(true);
            ch_ui.ampType->setEnabled(true);
            ch_ui.offsBox->setEnabled(true);
            ch_ui.freqBox->setEnabled(true);
            ch_ui.phaBox->setEnabled(true);
            ch_ui.dutyRatio->setEnabled(true);
        }
    }
}

void MainWindow::manualUpdateUi() {    
    Qt::CheckState en     = m_chUi[0].enBox->checkState();
    double freq = m_chUi[0].freqBox->value();    
    double amp  = m_chUi[0].ampBox->value();
    double offs = m_chUi[0].offsBox->value();
    double pha  = m_chUi[0].phaBox->value();
    double duty  = m_chUi[0].dutyRatio->value();
    int ch_type = m_chUi[0].typeBox->currentIndex();
    int ch_range = m_chUi[0].rangeBox->currentIndex();
    int amp_type = m_chUi[0].ampType->currentIndex();
    double amp_totalMax=0, amp_totalMin=0;
    double ch_totalMax=0, ch_totalMin=0;

    ui->dacFreqStd->setVisible(!ui->dacFreqManualEn->isChecked());
    ui->dacFreqManual->setVisible(ui->dacFreqManualEn->isChecked());

    for (int ch=0; ch < m_chUi.size(); ch++) {

        const double ch_max {m_curCfg->out().outDacChRangeMaxVal(ch)*1000};
        const double ch_min {m_curCfg->out().outDacChRangeMinVal(ch)*1000};

        double amp_max {ch_max};
        double amp_min {ch_min};
        QString type = m_chUi[ch].typeBox->currentText();
        if (type == LQMeas::AnalogSignalSin::instance().displayName()) {
            /* для синуса амплитуда всегда положительна.
             * если задается в rms, то диапазон доп. делим на корень из 2 */
            amp_min = 0;
            if (m_chUi[ch].ampType->currentData().toBool()) {
                amp_max = amp_max / sqrt(2);
            }
        }

        if ((ch == 0) || (ch_max > ch_totalMax))
            ch_totalMax = ch_max;

        if ((ch == 0) || (ch_min < ch_totalMin))
            ch_totalMin = ch_min;

        if ((ch == 0) || (amp_max > amp_totalMax))
            amp_totalMax = amp_max;

        if ((ch == 0) || (amp_min < amp_totalMin))
            amp_totalMin = amp_min;


        m_chUi[ch].ampBox->setMaximum(amp_max);
        m_chUi[ch].ampBox->setMinimum(amp_min);

        m_chUi[ch].offsBox->setMaximum(ch_max);
        m_chUi[ch].offsBox->setMinimum(ch_min);

        if (m_chUi[ch].enBox->checkState() != en)
            en = Qt::PartiallyChecked;
        if (m_chUi[ch].freqBox->value() != freq)
            freq = m_allChUi.freqBox->minimum();
        if (m_chUi[ch].phaBox->value() != pha)
            pha = m_chUi[ch].phaBox->minimum();
        if (m_chUi[ch].dutyRatio->value() != duty)
            duty = m_chUi[ch].dutyRatio->minimum();
        if (m_chUi[ch].ampBox->value() != amp)
            amp = m_allChUi.ampBox->minimum();
        if (m_chUi[ch].offsBox->value() != offs)
            offs = m_allChUi.offsBox->minimum();
        if (ch_type != m_chUi[ch].typeBox->currentIndex())
            ch_type = -1;
        if (ch_range != m_chUi[ch].rangeBox->currentIndex())
            ch_range = -1;
        if (amp_type != m_chUi[ch].ampType->currentIndex())
            amp_type = -1;
    }

    m_allChUi.enBox->setCheckState(en);
    m_allChUi.enBox->setTristate(en==Qt::PartiallyChecked);
    m_allChUi.freqBox->setValue(freq);
    m_allChUi.phaBox->setValue(pha);
    m_allChUi.dutyRatio->setValue(duty);
    m_allChUi.ampBox->setValue(amp);
    m_allChUi.offsBox->setValue(offs);
    m_allChUi.typeBox->setCurrentIndex(ch_type);
    m_allChUi.rangeBox->setCurrentIndex(ch_range);
    m_allChUi.ampType->setCurrentIndex(amp_type);

    m_allChUi.ampBox->setMaximum(amp_totalMax);
    m_allChUi.ampBox->setMinimum(amp_totalMin-0.00001);

    m_allChUi.offsBox->setMaximum(ch_totalMax);
    m_allChUi.offsBox->setMinimum(ch_totalMin-0.00001);


    for (int ch=0; ch < m_chUi.size(); ch++) {
        manualUpdateChUi(ch);
    }
    manualUpdateChUi(-1);

    ui->digSinglePulseCheck->setEnabled(ui->dacModeBox->currentIndex() == DAC_MODE_STREAM);
}

void MainWindow::saveSignalSettings() {
    QSettings set;
    set.beginGroup("Signal");
    set.setValue("dacFreqVal", LQMeas::LTR35OutConfig::typeStdGenFreqVal(
                     ui->dacFreqStd->currentData().value<LQMeas::LTR35OutConfig::OutStdGenFreq>()));
    set.setValue("dacFreqIsManual", ui->dacFreqManualEn->isChecked());
    set.setValue("dacFreqManualVal", ui->dacFreqManual->value());

    set.setValue("dacMode", ui->dacModeBox->currentIndex());
    set.setValue("syncStartMode", ui->syncStartMode->currentIndex());
    set.setValue("syncSlaveSrc", ui->slaveSyncSrc->currentIndex());
    set.setValue("echoEn", ui->echoEn->isChecked());
    set.setValue("echoBufSigTime", ui->echoBufSigTime->value());

    set.beginWriteArray("Channels", dac_ch_max_cnt);
    for (unsigned ch=0; ch < dac_ch_max_cnt; ch++) {
        set.setArrayIndex(ch);
        set.setValue("En",   m_chUi[ch].enBox->isChecked());
        set.setValue("Amp",  m_chUi[ch].ampBox->value()/1000);
        set.setValue("AmpType", m_chUi[ch].ampType->currentData().toBool() ? "RMS" : "Peak");
        set.setValue("Offs", m_chUi[ch].offsBox->value()/1000);
        set.setValue("Freq", m_chUi[ch].freqBox->value());
        set.setValue("Pha",  m_chUi[ch].phaBox->value());
        set.setValue("DutyRatio",  m_chUi[ch].dutyRatio->value());
        set.setValue("Type", m_chUi[ch].typeBox->currentIndex());
        set.setValue("Range", m_chUi[ch].rangeBox->currentIndex());
    }
    set.endArray();


    set.beginGroup("DigSigs");

    set.setValue("En", ui->digSigGroup->isChecked());
    set.setValue("Period", ui->digPeriod->value());
    set.setValue("PulseTime", ui->digPulseTime->value());
    set.setValue("SinglePulse", ui->digSinglePulseCheck->isChecked());

    set.endGroup();

    set.endGroup();
}

void MainWindow::loadSignalSettings() {
    QSettings set;
    QVariant val;
    set.beginGroup("Signal");


    ui->dacFreqManualEn->setChecked(set.value("dacFreqIsManual").toBool());
    ui->dacFreqManual->setValue(set.value("dacFreqManualVal", LTR35_DAC_QUAD_RATE_FREQ_STD).toDouble());
    double dac_freq = set.value("dacFreqVal", LTR35_DAC_QUAD_RATE_FREQ_STD).toDouble();
    int cur_idx = 0;
    for (int i = 0; i < ui->dacFreqStd->count(); i++) {
        if (qAbs(LQMeas::LTR35OutConfig::typeStdGenFreqVal(
                     ui->dacFreqStd->itemData(i).value<LQMeas::LTR35OutConfig::OutStdGenFreq>())
                 - dac_freq) < 0.1) {
            cur_idx = i;
        }
    }
    ui->dacFreqStd->setCurrentIndex(cur_idx);


    /*if ((val = set.value("dacFreq")).isValid()) {
        ui->dacFreqBox->setValue(val.toDouble());
        m_curCfg->outSyncSetGenFreq(val.toDouble());
    }*/

    if ((val = set.value("dacMode")).isValid())
        ui->dacModeBox->setCurrentIndex(val.toInt());

    if ((val = set.value("syncStartMode")).isValid())
        ui->syncStartMode->setCurrentIndex(val.toInt());

    if ((val = set.value("syncSlaveSrc")).isValid()) {
        ui->slaveSyncSrc->setCurrentIndex(val.toInt());
    }

    ui->echoEn->setChecked(set.value("echoEn", "1").toBool());
    if ((val = set.value("echoBufSigTime")).isValid()) {
        ui->echoBufSigTime->setValue(val.toInt());
    }

    set.beginReadArray("Channels");
    for (unsigned ch=0; ch < dac_ch_max_cnt; ch++) {
        set.setArrayIndex(ch);


        m_chUi[ch].enBox->setChecked(set.value("En", "1").toBool());
        if ((val = set.value("Amp")).isValid())
            m_chUi[ch].ampBox->setValue(val.toDouble()*1000);
        if ((val = set.value("AmpType")).isValid())
            m_chUi[ch].ampType->setCurrentIndex(val.toString() == "RMS" ? 1 : 0);
        if ((val = set.value("Offs")).isValid())
            m_chUi[ch].offsBox->setValue(val.toDouble()*1000);
        if ((val = set.value("Freq")).isValid())
            m_chUi[ch].freqBox->setValue(val.toDouble());
        if ((val = set.value("Pha")).isValid())
            m_chUi[ch].phaBox->setValue(val.toDouble());
        if ((val = set.value("DutyRatio")).isValid())
            m_chUi[ch].dutyRatio->setValue(val.toDouble());
        if ((val = set.value("Type")).isValid())
            m_chUi[ch].typeBox->setCurrentIndex(val.toInt());
        if ((val = set.value("Range")).isValid()) {
            m_chUi[ch].rangeBox->setCurrentIndex(val.toInt());
            m_curCfg->out().outDacSetChRangeNum(ch, val.toInt());
        }

    }
    set.endArray();

    set.beginGroup("DigSigs");

    ui->digSigGroup->setChecked(set.value("En", "1").toBool());
    if ((val = set.value("Period")).isValid())
        ui->digPeriod->setValue(val.toDouble());
    if ((val = set.value("PulseTime")).isValid())
        ui->digPulseTime->setValue(val.toDouble());

    ui->digSinglePulseCheck->setChecked(set.value("SinglePulse").toBool());

    set.endGroup();


    set.endGroup();
}

void MainWindow::allChEnChanged(int state) {
    if (state != Qt::PartiallyChecked) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].enBox->setCheckState((Qt::CheckState)state);
        }
        manualConnectSignals(true);
    }
    manualUpdateUi();

}

void MainWindow::allChPhaValChanged(double pha) {
    if (pha != m_allChUi.phaBox->minimum()) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].phaBox->setValue(pha);
        }
        manualConnectSignals(true);
    }
}

void MainWindow::allChFreqValChanged(double freq) {
    if (freq!=m_allChUi.freqBox->minimum()) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].freqBox->setValue(freq);
        }
        manualConnectSignals(true);
    }
}

void MainWindow::allChAmpTypeChanged(int type) {
    if (type >= 0) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].ampType->setCurrentIndex(type);
        }
        manualConnectSignals(true);
        manualUpdateUi();
    }
}

void MainWindow::allChAmpValChanged(double amp) {
    if (amp!=m_allChUi.ampBox->minimum()) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].ampBox->setValue(amp);
        }
        manualConnectSignals(true);
    }
}

void MainWindow::allChOffsValChanged(double offs) {
    if (offs!=m_allChUi.offsBox->minimum()) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].offsBox->setValue(offs);
        }
        manualConnectSignals(true);
    }
}

void MainWindow::allChDutyValChanged(double val) {
    if (val!=m_allChUi.dutyRatio->minimum()) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].dutyRatio->setValue(val);
        }
        manualConnectSignals(true);
    }
}

void MainWindow::allChSigTypeChanged(int ind) {
    if (ind >= 0) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].typeBox->setCurrentIndex(ind);
        }
        manualConnectSignals(true);
    }
    manualUpdateUi();
}

void MainWindow::allChRangeChanged(int ind) {
    if (ind >= 0) {
        manualConnectSignals(false);
        for (int ch=0; ch < m_chUi.size(); ch++) {
            m_chUi[ch].rangeBox->setCurrentIndex(ind);
            m_curCfg->out().outDacSetChRangeNum(ch, ind);
        }
        manualConnectSignals(true);
    }
    manualUpdateUi();
}

void MainWindow::rangeChanged(int idx) {
    QComboBox *box = qobject_cast<QComboBox*>(sender());
    if (box && (idx>=0)) {
        for (int ch=0; ch < m_curCfg->out().outInfo().outDacChannelsCnt(); ch++) {
            if (box == m_chUi[ch].rangeBox) {
                m_curCfg->out().outDacSetChRangeNum(ch, idx);
            }
        }
    }
    manualUpdateUi();
}

void MainWindow::manualConnectSignals(bool en) {
    for (int ch=0; ch < m_chUi.size(); ch++) {
        if (en) {
            connect(m_chUi[ch].enBox,      &QCheckBox::stateChanged, this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].freqBox,    static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].phaBox,     static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].dutyRatio,  static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].ampBox,     static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].offsBox,    static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].typeBox,    static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
                    this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].rangeBox,   static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
                    this, &MainWindow::manualUpdateUi);
            connect(m_chUi[ch].ampType,    static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
                    this, &MainWindow::manualUpdateUi);
        } else {
            disconnect(m_chUi[ch].enBox,      &QCheckBox::stateChanged, this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].freqBox,    static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].phaBox,     static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].dutyRatio,  static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].ampBox,     static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].offsBox,    static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                    this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].typeBox,    static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
                    this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].rangeBox,   static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
                    this, &MainWindow::manualUpdateUi);
            disconnect(m_chUi[ch].ampType,    static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
                    this, &MainWindow::manualUpdateUi);
        }
    }

}

void MainWindow::onSetSigButtonClicked() {
    setSignals();
}

void MainWindow::setSignals(bool resetMode) {
    QSharedPointer<LQMeas::LTR35> dac = curDev();
    int dac_mode = ui->dacModeBox->currentIndex();
    QString err_msg;
    bool dig_single_impulse = dac_mode == DAC_MODE_STREAM ? ui->digSinglePulseCheck->isChecked() : false;


    if (dac) {
        LQError err = LQError::Success();



        WaitDialog dialog(this);
        dialog.setModal(true);
        dialog.show();
        unsigned ch_cnt = 0;


        if (dac->outSyncGenRunning()) {
            LQError stop_err;
            dialog.setText(tr("Остановка генерации..."));
            QCoreApplication::processEvents();

            dac->outSyncGenStopRequest(stop_gen_tout, stop_err);
            dac->outSyncGenStop(stop_err);
        }

        dialog.setText(tr("Установка сигналов..."));
        ui->SignalLed->setChecked(false);
        QCoreApplication::processEvents();

        m_curCfg->out().outSyncSetManualGenFreqEnabled(ui->dacFreqManualEn->isChecked());
        m_curCfg->out().outSyncSetManualGenFreq(ui->dacFreqManual->value());
        m_curCfg->out().outSyncSetStdGenFreq(ui->dacFreqStd->currentData().value<LQMeas::LTR35OutConfig::OutStdGenFreq>());

        m_curCfg->out().outSetSyncGenModeEnabled(true);
        m_curCfg->out().outDacSetAfcCorEnabled(m_afcCfg.enabled());
        m_curCfg->sync().setSyncStartMode(
                    dac_mode == DAC_MODE_STREAM ? ui->syncStartMode->currentData().value<LQMeas::LTR35SyncConfig::SyncStartMode>()
                                                : LQMeas::LTR35SyncConfig::SyncStartMode::Internal);
        m_curCfg->sync().setSyncSlaveSource(ui->slaveSyncSrc->currentData().value<LQMeas::LTR35SyncConfig::SyncSlaveSrc>());
        m_curCfg->out().outSyncSetEchoEnabled((dac_mode == DAC_MODE_STREAM) && ui->echoEn->isChecked());
        m_curCfg->out().outSyncSetPreloadSignalTimeMs(ui->echoBufSigTime->value());

        int first_ch = -1;


        for (int ch=0; ch < LQMeas::LTR35OutConfig::max_dac_ch_cnt; ++ch) {
            if (ch < m_curCfg->out().outInfo().outDacChannelsCnt()) {
                QSharedPointer<LQMeas::SignalConfig> sig;
                bool en = !resetMode && m_chUi[ch].enBox->isChecked();
                m_curCfg->out().outSyncDacSetChEnabled(ch, en);
                if (en) {
                    if (first_ch < 0)
                        first_ch = ch;
                    ch_cnt++;

                    double amp = m_chUi[ch].ampBox->value()/1000;
                    double offs = m_chUi[ch].offsBox->value()/1000;
                    double freq = m_chUi[ch].freqBox->value();
                    double pha = m_chUi[ch].phaBox->value();
                    bool amp_rms = m_chUi[ch].ampType->currentData().toBool();

                    m_curCfg->out().outSyncDacSetChGenMode(ch, dac_mode == DAC_MODE_ATITH ?
                                                     LQMeas::DevOutSyncConfig::OutSyncChGenMode::Hard :
                                                     LQMeas::DevOutSyncConfig::OutSyncChGenMode::Ram);
                    QString type = m_chUi[ch].typeBox->currentText();
                    if (type == LQMeas::AnalogSignalSin::instance().displayName()) {
                        sig = QSharedPointer<LQMeas::SignalConfig>(new LQMeas::AnalogSignalSinConfig(
                                                            amp_rms ? amp * sqrt(2) : amp,
                                                            offs,
                                                            freq,
                                                            pha*M_PI/180.
                                                            ));
                    } else if (type == LQMeas::AnalogSignalConst::instance().displayName()) {
                        sig = QSharedPointer<LQMeas::SignalConfig>(new LQMeas::AnalogSignalConstConfig(amp));
                        if (dac_mode == DAC_MODE_ATITH) {
                            err = LQMeas::StdErrors::DeviceUnsupConfig();
                            err_msg = tr("В арифметическом режиме можно установить только синусоидальный сигнал");
                        }
                    } else if (type == LQMeas::AnalogSignalPulse::instance().displayName()) {
                        double duty = 1./m_chUi[ch].dutyRatio->value(); /* Duty Cycle = 1./скважность */


                        sig = QSharedPointer<LQMeas::SignalConfig>(new LQMeas::AnalogSignalPulseConfig(
                                                                       offs + amp,
                                                                       offs - amp,
                                                                       freq,
                                                                       duty));
                        if (dac_mode == DAC_MODE_ATITH) {
                            err = LQMeas::StdErrors::DeviceUnsupConfig();
                            err_msg = tr("В арифметическом режиме можно установить только синусоидальный сигнал");
                        }
                    }
                    dac->outSyncDacSetGenSignal(ch, *sig, err);
                }
            } else {
                m_curCfg->out().outSyncDacSetChEnabled(ch, false);
            }
        }
        m_curCfg->out().outSyncSetEchoChannel(first_ch);



        if (err.isSuccess()) {
            QSharedPointer<LQMeas::SignalConfig> digSig;
            double t_sig = ui->digPeriod->value() / 1000000;
            double t_pulse = ui->digPulseTime->value() / 1000000;
            if (ui->digSigGroup->isChecked() && !resetMode) {
                ch_cnt++;
                if (!dig_single_impulse) {
                    digSig = QSharedPointer<LQMeas::SignalConfig>(new LQMeas::DiscreteSignalPulseConfig(1./t_sig, t_pulse/t_sig));
                } else {
                    digSig = QSharedPointer<LQMeas::SignalConfig>(new LQMeas::DiscreteSignalSinglePulseBurstConfig(1./t_sig, t_pulse/t_sig));
                }

                for (int ch=0; (ch < m_curCfg->out().outInfo().outDigChannelsCnt()) && err.isSuccess(); ch++) {
                    dac->outSyncDigSetGenSignal(ch, *digSig, err);
                }
            }

            for (int ch=0; (ch < m_curCfg->out().outInfo().outDigChannelsCnt()) && err.isSuccess(); ch++) {
                m_curCfg->out().outSyncDigSetChEnabled(ch, digSig);
            }


            if (dac_mode == DAC_MODE_STREAM) {
                m_curCfg->update();
                m_curCfg->out().outSyncSetRamMode(LQMeas::DevOutSyncConfig::OutSyncRamMode::Stream);
                if ((ch_cnt * m_curCfg->out().outSyncGenFreq()) > 500000) {
                    err = LQMeas::StdErrors::DeviceUnsupConfig();
                    err_msg = tr("Превышено максимальное колчество каналов в потоковом режиме!");
                }
            } else {
                m_curCfg->out().outSyncSetRamMode(LQMeas::DevOutSyncConfig::OutSyncRamMode::Cycle);
            }

            if (dac_mode == DAC_MODE_ATITH) {

                QList<double> arith_freqs;
                QList<double> arith_phases;

                for (int ch=0; (ch < m_curCfg->out().outInfo().outDacChannelsCnt()) && err.isSuccess(); ch++) {
                    bool en = m_chUi[ch].enBox->isChecked();
                    if (en) {
                        const double freq {m_chUi[ch].freqBox->value()};
                        const double pha {m_chUi[ch].phaBox->value()};
                        int src = -1;
                        bool cos_mode = false;
                        /* пробуем найти арифметический источник с такой же частотой */
                        for (int i=0; i < arith_freqs.size(); i++) {
                            if ((qAbs(freq-arith_freqs.at(i)) < 0.001)
                                    && ((qAbs(pha - arith_phases.at(i)) < 0.01)
                                        || (qAbs(pha - arith_phases.at(i) - 90) < 0.01))) {
                                src = i;
                                cos_mode = qAbs(pha - arith_phases.at(i) - 90) < 0.01;
                            }
                        }

                        /* если не нашли источник, значит новый, но может
                         * быть не больше LTR35_ARITH_SRC_CNT */
                        if (src < 0) {
                            if (arith_freqs.size() != LTR35_ARITH_SRC_CNT) {
                                src = arith_freqs.size();
                                arith_freqs.append(freq);
                                arith_phases.append(pha);
                            } else {
                                dac->getError(LTR35_ERR_UNSUPPORTED_CONFIG, err);
                            }
                        }

                        if (err.isSuccess()) {
                            m_curCfg->out().outDacChSetHardSourceNumber(ch, src);
                            m_curCfg->out().outDacChSetHardSourceOutput(
                                        ch, cos_mode ? LQMeas::LTR35OutConfig::HardSrcOutput::Cos :
                                                       LQMeas::LTR35OutConfig::HardSrcOutput::Sin);
                        }
                    }
                }

                for (int hard_src_idx = 0; (hard_src_idx < arith_freqs.size()) && err.isSuccess(); ++hard_src_idx) {
                    dac->setHardGenParams(hard_src_idx, arith_freqs.at(hard_src_idx), arith_phases.at(hard_src_idx), err);
                }
            }
        }

        if (err.isSuccess())
            dac->configure(*m_curCfg, err);

        bool done = false;
        if (err.isSuccess()) {
            if (ch_cnt > 0) {
                if (m_curCfg->sync().syncStartMode() == LQMeas::LTR35SyncConfig::SyncStartMode::Slave) {
                    dialog.setText(tr("Ожидание сигнала синхронного старта"), true);
                    dac->outSyncGenStartRequest(err);
                    if (err.isSuccess()) {

                        while (err.isSuccess() && !done && !dialog.isCanceled())  {
                            qApp->processEvents();
                            done = dac->outSyncGenStartWaitDone(50, err);
                        }
                    }

                    if (!done) {
                        LQError stop_err;
                        dac->outSyncGenStopRequest(stop_gen_tout, stop_err);
                        dac->outSyncGenStop(stop_err);
                    }
                } else {
                    dialog.setText(tr("Запуск генерации"));
                    dac->outSyncGenStart(err);
                    done = err.isSuccess();
                }
            } else {
                done = true;
            }
        }

        dialog.close();
        if (err.isSuccess()) {
            if (done) {
                for (int ch=0; ch < m_curCfg->out().outInfo().outDacChannelsCnt(); ch++) {
                    if (m_curCfg->out().outSyncDacChEnabled(ch)) {
                        if (dac->outSyncDacGenSignalIsValid(ch)) {
                            const LQMeas::SignalConfig &sig {dac->outSyncDacGenSignal(ch)};
                            if (sig.signalFreq() > 0)
                                m_chUi[ch].freqBox->setValue(sig.signalRealFreq());
                        }
                    }
                }

                if (m_curCfg->out().outSyncDigChEnabled(0)) {
                    if (dac->outSyncDigGenSignalIsValid(0)) {
                        const LQMeas::SignalConfig &digSig {dac->outSyncDigGenSignal(0)};
                        double real_t {1000000./digSig.signalRealFreq()};
                        ui->digPeriod->setValue(real_t);
                        if (!dig_single_impulse) {
                            ui->digPulseTime->setValue(real_t*static_cast<const LQMeas::DiscreteSignalPulseConfig &>
                                                       (digSig).realDutyCycle());
                        } else {
                            ui->digPulseTime->setValue(real_t*static_cast<const LQMeas::DiscreteSignalSinglePulseBurstConfig &>
                                                       (digSig).realDutyCycle());
                        }
                    }
                }

                LQMeasLog->info(tr("Сигнал установлен успешно!"));

                if (ui->dacFreqManualEn->isChecked())
                    ui->dacFreqManual->setValue(dac->devspecConfig().out().outSyncGenFreq());
            }

            ui->SignalLed->setChecked(done && (ch_cnt > 0));
        } else {
            QMessageBox::critical(this, tr("Ошибка установки сигнала"),
                                  tr("Не удалось установить сигнал: ") +
                                  (err_msg.isEmpty() ? err.msg() : err_msg));
        }
    }
}

void MainWindow::disableChannels() {
    setSignals(true);
}

void MainWindow::showConfigDialog() {
    ConfigDialog dlg(this, m_afcCfg, curDev().data());
    if (dlg.exec() == QDialog::Accepted) {
        dlg.fillConfig(m_afcCfg);
        applyAfcParams();
        if (ui->SignalLed->isChecked()) {
            setSignals();
        }
    }
}

void MainWindow::uiEnStateUpdate() {
    const int dac_mode {ui->dacModeBox->currentIndex()};
    const bool stream_mode {dac_mode == DAC_MODE_STREAM};
    ui->syncStartMode->setEnabled(stream_mode);
    ui->slaveSyncSrc->setEnabled(stream_mode &&
                                 ui->syncStartMode->currentData().value<LQMeas::LTR35SyncConfig::SyncStartMode>()
                                 == LQMeas::LTR35SyncConfig::SyncStartMode::Slave);
    ui->echoEn->setEnabled(stream_mode);
    ui->echoBufSigTime->setEnabled(stream_mode && ui->echoEn->isChecked());
}

void MainWindow::closeEvent(QCloseEvent *event) {
    saveSignalSettings();

    QSettings set;
    set.beginGroup("DeviceConfig");
    set.beginGroup("Afc");
    m_afcCfg.save(set);
    set.endGroup();
    set.endGroup();


    set.beginGroup("MainWindow");
    set.setValue("state", saveState());
    set.setValue("size", size());
    set.setValue("pos", pos());
    set.endGroup();



    closeCurModule();
}

void MainWindow::applyAfcParams() {
    QSharedPointer<LQMeas::LTR35> dac = curDev();
    if (dac) {
        if (m_afcCfg.isManual()) {
            TLTR35_AFC_COEF *afcCoefs {&dac->rawHandle()->ModuleInfo.CbrAfcCoef};
            afcCoefs->Valid = TRUE;
            afcCoefs->SigFreq = m_afcCfg.f0();
            afcCoefs->Fd = m_afcCfg.fd0();
            for (int ch_idx = 0; ch_idx < (int)dac_ch_max_cnt; ch_idx++) {
                afcCoefs->K[ch_idx] = m_afcCfg.k(ch_idx);
            }
        } else {
            LTR35_GetConfig(dac->rawHandle());
        }
    }
}

QSharedPointer<LQMeas::LTR35> MainWindow::curDev() {
    return qSharedPointerObjectCast<LQMeas::LTR35>(m_devsel->currentModule());
}
