#ifndef DEVADCSEQBASECONFIG_H
#define DEVADCSEQBASECONFIG_H


#include "DevAdcConfig.h"
#include <QVector>

namespace LQMeas {
    /* Вспомогательный класс, реализующих установку стандартных параметров
     * для АЦП с коммутацией каналов на 32 канала с общей землей/16 дифференциальных
     * каналов с настраиваемым диапазоном для каждого канала */
    class DevAdcSeqBaseConfig : public DevAdcConfig {
        Q_OBJECT
    public:
        DevAdcSeqBaseConfig(const DevAdcInfo &info);
        DevAdcSeqBaseConfig(const DevAdcSeqBaseConfig &cfg);
        ~DevAdcSeqBaseConfig() override;

        /* результирующие параметры, действительные только после adcUpdateConfig() */
        double adcFreq() const override {return m_results.AdcFreq;}
        double adcChFreq() const override {return m_results.AdcChFreq;}
        /* изначально установленные параметры частоты */
        double adcConfiguredFreq() const {return m_adc_params.FreqAdc;}
        double adcConfiguredChFreq() const {return m_adc_params.FreqCh;}
        bool adcUseMaxChFreq() const {return m_adc_params.UseMaxChFreq;}

        bool adcChEnabled(int ch) const override { return m_ch_enabled[ch]; }


    public Q_SLOTS:
        void adcSetChEnabled(int ch, bool en);
        void adcSetFreq(double freq);
        void adcSetChFreq(double ch_freq);
        void adcSetUseMaxChFreq(bool en);
    protected:
        void protUpdate() override;

        void protSave(QJsonObject &cfgObj) const override;
        void protLoad(const QJsonObject &cfgObj) override;

        virtual void adcAdjustFreq(double *adcFreq, double *adcChFreq) = 0;
        virtual void adcChSave(int ch, QJsonObject &chObj) const = 0;
        virtual void adcChLoad(int ch, const QJsonObject &chObj) = 0;
        virtual void adcChSetDefault(int ch) = 0;
    private:
        int m_ch_cnt;
        bool *m_ch_enabled;
        struct {
            double FreqAdc;
            double FreqCh;
            bool   UseMaxChFreq;
        } m_adc_params;

        struct {
            double AdcFreq;
            double AdcChFreq;
        } m_results;
    };
}



#endif // DEVADCSEQBASECONFIG_H
