#ifndef DEVADCSTDSEQ32CONFIG_H
#define DEVADCSTDSEQ32CONFIG_H

#include "DevAdcSeqBaseConfig.h"

namespace LQMeas {
    /* Вспомогательный класс, реализующих установку стандартных параметров
     * для АЦП с коммутацией каналов на 32 канала с общей землей/16 дифференциальных
     * каналов с настраиваемым диапазоном для каждого канала */
    class DevAdcStdSeq32Config : public DevAdcSeqBaseConfig {
        Q_OBJECT
    public:
        static const int adc_channels_cnt = 32;
        static const int adc_diff_channels_cnt = 16;

        enum class AdcChMode {
            Comm  = 0,
            Diff = 1,
            Zero = 2
        };


        DevAdcStdSeq32Config(const DevAdcInfo &info);
        DevAdcStdSeq32Config(const DevAdcStdSeq32Config &cfg);

        int adcChRangeNum(int ch) const {return m_ch_params[ch].Range;}
        AdcChMode adcChMode(int ch) const {return m_ch_params[ch].Mode;}

        bool adcHasZeroChannels() const;
    public Q_SLOTS:
        void adcSetChMode(int ch, AdcChMode mode);
        void adcSetChRangeNum(int ch, int range_num);
    protected:
        void adcChSave(int ch, QJsonObject &chObj) const;
        void adcChLoad(int ch, const QJsonObject &chObj);
        void adcChSetDefault(int ch);
    private:
        struct {
            AdcChMode Mode;
            int Range;
        } m_ch_params[adc_channels_cnt];
    };
}


#endif // DEVADCSTDSEQ32CONFIG_H
